----------------------------------------------------------------------------------
--  Autor: Victor Ral Arteaga Prez
--
-- Description: Este mdulo se encarga de controlar al DAC generando la interfaz SPI.
--              La frecuencia de muestreo se cambia dependiendo del valor que se haya recibido
--              del ESP32, con eso se logra cambiar la frecuencia de la seal.
--
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity DAC_Demo is

Port ( 
    CLK     : IN  STD_LOGIC; -- Reloj del FPGA a 100MHz
    RESET   : IN  STD_LOGIC; -- Resetea la transmisin SPI
    SDI     : OUT STD_LOGIC; -- Puerto serial para SPI
    CS      : OUT STD_LOGIC; -- Chip Select del MCP4822
    SCK     : OUT STD_LOGIC; -- Reloj del protocolo SPI
    LDAC    : OUT STD_LOGIC; -- Pin LDAC del MCP4822
    FREC    : IN  STD_LOGIC_VECTOR(7  DOWNTO 0); -- Puerto de entrada para modificar la frecuencia de la funcin generada
    FUN     : IN  STD_LOGIC_VECTOR(11 DOWNTO 0); -- Puerto que selecciona la funcin a generar (direccin de ROM "MEM_MUESTRAS")
    DATA_IN : IN  STD_LOGIC_VECTOR(11 DOWNTO 0)  -- Valor digital a convertir de 12 bits
);

end DAC_Demo;

architecture Behavioral of DAC_Demo is

component MEM_MUESTRAS IS
  PORT (
    a : IN integer;
    spo : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
  );
END component MEM_MUESTRAS;

-- Mquina de estados que controla el DAC
type state is (idle,tx_rx,fin);
signal pr_state, nx_state: state;
-- Seales para mquina de estados
signal clk_ms     : std_logic := '0'; -- Genera seal de reloj
signal buffer_out : std_logic_vector(15 downto 0) := (others => '0'); --Buffer que contiene los datos a transmitir al DAC
signal spo        : STD_LOGIC_VECTOR(15 DOWNTO 0); -- Contiene el dato de la muestra, slo se ocupan los 12 bits menos significativos
signal i          : integer range 0 to 31   := 0;  -- Indica el bit del buffer que se transmitir
signal bits       : integer range 0 to 31   := 1;  -- Indica el nmero de bits que se transmitirn
signal senal      : integer range 0 to 4095 := 0;  -- Registro que almacena la muestra para mandarla al buffer
signal a          : INTEGER; -- Indica la direccin de la memoria ROM a leer

begin

u1: MEM_MUESTRAS
  PORT map(
    a,
    spo
  );

-- Cargamos el buffer con los datos
buffer_out <= "0011" & std_logic_vector(to_unsigned(senal,12));

-- Proceso que genera la frecuencia de muestreo del DAC
-- Cuando "contador" llega a su valor mximo entonces se manda la siguiente muestra
process(clk)
variable contador: integer range 0 to 65535;
begin
if rising_edge(clk) then
contador := contador + 1;
	if(contador>=3*(255-(CONV_INTEGER(FREC)))) then-- El valor mximo del contador vara dependiendo del byte que se haya recibido del ESP32
        clk_ms<=not clk_ms;
        contador := 0;
	end if;
end if;
end process;


-- Proceso que hace el barrido de los bits del buffer y tambin selecciona la
-- direccin de la memoria ROM para tomar el valor de la muestra.
process(clk_ms,reset)
begin
		
if reset = '1' then
pr_state <= idle;
i <= 0;
elsif rising_edge(clk_ms) then

	if (i = bits - 1) then -- Espera a que se hayan enviado todos los bits
		pr_state <= nx_state;
		i <= 0;		
	else
		i <= i + 1;
	end if;
	
	if pr_state = fin then -- Controla la direccin de la memoria ROM
        SENAL <= conv_integer(SPO);
        a <= a+1;
	if a >= 159+CONV_INTEGER(FUN) then
	   a <= CONV_INTEGER(FUN);
	end if;
    end if;

end if;

end process;


-- Proceso que genera el protocolo SPI para comunicarse con el MCP4822
process(pr_state,clk_ms,i)
begin

 case pr_state is
 
  when idle => --estado neutral 
	  SCK <= '1';
	  SDI <= '0';
	  LDAC <= '1';
	  CS <= '0';
	  bits <= 1;	  
	  nx_state <= tx_rx;
 
	when tx_rx =>
	  SCK <= not clk_ms;
	  SDI <= buffer_out(15 - i);
	  LDAC <= '1';
      CS <= '0';
	  bits <= 16;
	  nx_state <= fin;
	  
	when fin =>
	  SCK <= '1';
	  SDI <= '0';
	  LDAC <= '0';
      CS <= '1';
	  bits <= 1;	 
	  nx_state <= idle;
  
 end case;
end process;



end Behavioral;
