----------------------------------------------------------------------------------
--  Autor: Jess Eduardo Mndez Rosales
--
-- Description: Aplicacin demostrativa que muestra un ejemplo de adquisicin y transmisin de datos 
--              a travs de un Websocket con ESP32 creado en ESP-IDF. Adems hace lecturas en los 7 canales
--              del ADC, as como generacin de funciones con el DAC con frecuencia variable.  
--              El ESP32 utiliza el UART para realizar la comunicacin con el FPGA a 115200 baudios. La secuencia
--              de racepcin y transmisin de datos es la siguiente:
--
--                - El ESP32 manda un byte con la funcin a generar (Se pueden generar hasta 5 funciones)
--                - El ESP32 manda un byte con la frecuencia.
--                - El FPGA manda 7 bytes con la informacin de los 7 canales del ADC. Slo se estn tomando los 8 bits
--                  ms significativos del valor digital de la conversin.
--                      				
--
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity TOP is

Port (
       CLK       : IN  STD_LOGIC; -- Reloj FPGA a 100Mhz
       TX_FTDI   : IN  STD_LOGIC; -- Pin de transmisin del FTDI
       PTX_ESP32 : IN  STD_LOGIC; -- Pin conectado a TX  para programar el ESP32
       TX_ESP32  : IN  STD_LOGIC; -- Pin conectado al TX del ESP32
       SW_BOOT   : IN  STD_LOGIC; -- Switch para controlar al boot del ESP32
       SW_EN     : IN  STD_LOGIC; -- Switch para controlar a EN del ESP32 
       adc_p     : in  std_logic_vector(6 downto 0); -- Canales diferenciales 
       adc_n     : in  std_logic_vector(6 downto 0); -- del ADC
       RX_FTDI   : OUT STD_LOGIC; -- Pin de recepcin del FTDI
       PRX_ESP32 : OUT STD_LOGIC; -- Pin conectado a RX para programar el ESP32
       RX_ESP32  : OUT STD_LOGIC; -- Pin conectado al RX del ESP32
       SDI       : OUT STD_LOGIC; -- Puerto serial de SPI para DAC
       SCLK      : OUT STD_LOGIC; -- Reloj del SPI
       CS        : OUT STD_LOGIC; -- Chip Select del SPI
       LDAC      : OUT STD_LOGIC; -- LDAC de MCP4822
       ESP_BOOT  : OUT STD_LOGIC; -- Puerto conectado al boot del ESP32 
       ESP_EN    : OUT STD_LOGIC  -- Puerto conectado al EN del ESP322
        
 );

end TOP;

architecture Behavioral of TOP is

COMPONENT RS232 is
generic	(	FPGA_CLK   : INTEGER := 50000000; 
			BAUD_RS232 : INTEGER := 9600
			);
port	(	CLK    : in  std_logic;
			RX     : in  std_logic;
			TX_INI : in  std_logic;
			TX_FIN : out std_logic;
			TX     : out std_logic;
			RX_IN  : out std_logic; 
			DATAIN : in  std_logic_vector(7 downto 0);
			DOUT   : out std_logic_vector(7 downto 0) 
		);
end COMPONENT RS232; 

COMPONENT DAC_Demo is
Port ( 
    CLK     : IN  STD_LOGIC;
    RESET   : IN  STD_LOGIC;
    SDI     : OUT STD_LOGIC;
    CS      : OUT STD_LOGIC;
    SCK     : OUT STD_LOGIC;
    LDAC    : OUT STD_LOGIC;
    DATA_IN : IN  STD_LOGIC_VECTOR(11 DOWNTO 0);
    FREC    : IN  STD_LOGIC_VECTOR(7  DOWNTO 0); 
    FUN     : IN  STD_LOGIC_VECTOR(11 DOWNTO 0)
);
END COMPONENT DAC_Demo;

COMPONENT ADC is

PORT(
    CLK   : IN  STD_LOGIC;
    ADC_P : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
    ADC_N : IN  STD_LOGIC_VECTOR(6 DOWNTO 0);
    CH    : IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
    DATA_ADC  : OUT STD_LOGIC_VECTOR(11 DOWNTO 0)
    );
    
end COMPONENT ADC;

-- Seales para librera RS232
signal	tx_ini : std_logic := '0';
signal	tx_fin : std_logic := '0';
signal	rx_in  : std_logic := '0'; 
signal	datain : std_logic_vector(7 downto 0) := (others => '0');
signal	dout   : std_logic_vector(7 downto 0) := (others => '0');

-- Seales para el componente DAC_demo
CONSTANT RESET   : STD_LOGIC := '0';
signal   data_in : std_logic_vector(11 downto 0) := (others => '0');
signal   frec    : std_logic_vector(7  downto 0) := (others => '0');
signal   fun     : std_logic_vector(11 downto 0) := (others => '0');

-- Seales para el componente ADC
signal ch       : std_logic_vector(3  downto 0) := (others => '0');
signal data_adc : std_logic_vector(11 downto 0) := (others => '0');

-- Seales para mquina de estados
CONSTANT X      : INTEGER := 2000;             -- Constante para generar un retardo de 20us para la transmisin del UART entre byte y byte
CONSTANT FINADC : INTEGER := 1_000_000;        -- Constante para generar un retardo de 10ms para esperar la conversin del ADC
signal r        : integer range 0 to X   := 0; -- Contador para el retardo de 20us 
signal edo      : integer range 0 to 100 := 0; -- Seal para mquina de estados que controla recepcin y transmisin
signal contaadc : integer range 0 to FINADC := 0;-- Conador para generar retardo de 10ms



begin

U1_RS232: RS232 generic MAP(FPGA_CLK => 100000000, 
				BAUD_RS232=> 115200
			)
port MAP(	CLK,
			TX_ESP32,
			TX_INI,
			TX_FIN,
			RX_ESP32,
			RX_IN, 
			DATAIN,
			DOUT
		);

  
U2_DAC: DAC_Demo
        Port MAP( 
        clk,
        reset,
        SDI,
        CS,
        SCLK,
        LDAC,
        DATA_IN,
        FREC,
        FUN
        );
        
 U3_ADC : ADC
        
       PORT MAP(
            CLK,
            ADC_P,
            ADC_N,
            CH,
            DATA_ADC
            );
        

-- Conectando el FTDI con el ESP32
PRX_ESP32 <= TX_FTDI;
RX_FTDI <= PTX_ESP32;

-- Conectando los switches de Avanxe7 al ESP32
ESP_BOOT <= SW_BOOT;
ESP_EN <= SW_EN;


PROCESS(CLK)
BEGIN
IF RISING_EDGE(CLK) THEN
    IF EDO = 0 THEN
        -- El priber byte que se recibe contiene la funcin a generar
        IF RX_IN = '1' THEN
            IF DOUT = X"0A" THEN--Seno
                FUN <= X"000";
                EDO <= 1;
            ELSIF DOUT = X"14" THEN--Triangular
                FUN <= "000010100000";
                EDO <= 1;
            ELSIF DOUT = X"1E" THEN--Cuadrada
                FUN <= "000101000000";
                EDO <= 1;
            ELSIF DOUT = X"28" THEN--Diente de sierra
                FUN <= "000111100000";
                EDO <= 1;
            ELSIF DOUT = X"32" THEN--Random
                FUN <= "001010000000";
                EDO <= 1;
            END IF;
        ELSE
            EDO <= 0;
        END IF;

    ELSIF EDO = 1 THEN
        --El segundo byte que se recibe es la frecuencia de la funcin
        IF RX_IN = '1' THEN
            FREC <= DOUT;
            EDO <= 2;
        ELSE
            EDO <= 1;
        END IF;

----------ADC0--------------
    ELSIF EDO = 2 THEN
        --Se selecciona el canal del ADC
        CH <= X"0";
        EDO <= 3; 

    ELSIF EDO = 3 THEN
        --Generamos un retardo de 10ms para esperar la conversin del ADC
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 4;
        else
            edo <= 3;
        end if;    

    ELSIF EDO = 4 THEN
        --Se mandan los 8 bits ms significativos de la conversin al UART del ESP32
        IF TX_FIN = '1' THEN
            EDO <= 5;
            TX_INI <= '0';
        ELSE
            EDO <= 4;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 5 THEN
        --Generamos retardo de 20us para evitar desincronizacin con el ESP32
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 6;
        ELSE
            EDO <= 5;
        END IF;
----------------------------  

-- El proceso que se hizo para el canal 0 se repite para los dems canales.

----------ADC1--------------
    ELSIF EDO = 6 THEN
        CH <= X"1";
        EDO <= 7; 

    ELSIF EDO = 7 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 8;
        else
            edo <= 7;
        end if;    

    ELSIF EDO = 8 THEN
        IF TX_FIN = '1' THEN
            EDO <= 9;
            TX_INI <= '0';
        ELSE
            EDO <= 8;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 9 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 10;
        ELSE
            EDO <= 9;
        END IF;
    ----------------------------

----------ADC2--------------
    ELSIF EDO = 10 THEN
        CH <= X"2";
        EDO <= 11; 

    ELSIF EDO = 11 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 12;
        else
            edo <= 11;
        end if;    

    ELSIF EDO = 12 THEN
        IF TX_FIN = '1' THEN
            EDO <= 13;
            TX_INI <= '0';
        ELSE
            EDO <= 12;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 13 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 14;
        ELSE
            EDO <= 13;
        END IF;
----------------------------

----------ADC3--------------
    ELSIF EDO = 14 THEN
        CH <= X"3";
        EDO <= 15; 

    ELSIF EDO = 15 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 16;
        else
            edo <= 15;
        end if;    

    ELSIF EDO = 16 THEN
        IF TX_FIN = '1' THEN
            EDO <= 17;
            TX_INI <= '0';
        ELSE
            EDO <= 16;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 17 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 18;
        ELSE
            EDO <= 17;
        END IF;
----------------------------

----------ADC4--------------
    ELSIF EDO = 18 THEN
        CH <= X"4";
        EDO <= 19; 

    ELSIF EDO = 19 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 20;
        else
            edo <= 19;
        end if;    

    ELSIF EDO = 20 THEN
        IF TX_FIN = '1' THEN
            EDO <= 21;
            TX_INI <= '0';
        ELSE
            EDO <= 20;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 21 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 22;
        ELSE
            EDO <= 21;
        END IF;
----------------------------

----------ADC5--------------
    ELSIF EDO = 22 THEN
        CH <= X"5";
        EDO <= 23; 

    ELSIF EDO = 23 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 24;
        else
            edo <= 23;
        end if;    
    
    ELSIF EDO = 24 THEN
        IF TX_FIN = '1' THEN
            EDO <= 25;
            TX_INI <= '0';
        ELSE
            EDO <= 24;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 25 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 26;
        ELSE
            EDO <= 25;
        END IF;
----------------------------

----------ADC6--------------
    ELSIF EDO = 26 THEN
        CH <= X"6";
        EDO <= 27; 

    ELSIF EDO = 27 THEN
        contaADC <= contaADC+1;
        if contaADC = finADC then
            contaADC <= 0;
            edo <= 28;
        else
            edo <= 27;
        end if;    

    ELSIF EDO = 28 THEN
        IF TX_FIN = '1' THEN
            EDO <= 29;
            TX_INI <= '0';
        ELSE
            EDO <= 28;
            DATAIN <= DATA_ADC(11 DOWNTO 4);
            TX_INI <= '1';
        END IF;

    ELSIF EDO = 29 THEN
        R <= R+1;
        IF R = X THEN
            R <= 0;
            EDO <= 0;
        ELSE
            EDO <= 29;
        END IF;

    END IF;
END IF;
END PROCESS;

end Behavioral;
